@@ -18,7 +18,7 @@
    of multiple items from multiple sources as well as dynamic balancing
    of load between the sources.
    
-   Schedualing of downloads is done on a first ask first get basis. This
+   Scheduling of downloads is done on a first ask first get basis. This
    preserves the order of the download as much as possible. And means the
    fastest source will tend to process the largest number of files.
    
@@ -69,6 +69,7 @@
 #include <apt-pkg/macros.h>
 #include <apt-pkg/weakptr.h>
 
+#include <chrono>
 #include <string>
 #include <vector>
 
@@ -92,6 +93,10 @@ class metaIndex;
 class APT_PUBLIC pkgAcquire
 {   
    private:
+   /** \brief The monotonic clock used by the Acquire system */
+   using clock = std::chrono::steady_clock;
+   /** \brief Time point on our monotonic clock */
+   using time_point = std::chrono::time_point<clock>;
    /** \brief FD of the Lock file we acquire in Setup (if any) */
    int LockFD;
    /** \brief dpointer placeholder (for later in case we need it) */
@@ -458,6 +463,8 @@ class APT_PUBLIC pkgAcquire::Queue
       std::string Custom600Headers() const;
       /** @return the maximum priority of this item */
       int APT_HIDDEN GetPriority() const;
+      /** @return the maximum time to fetch this item at */
+      time_point APT_HIDDEN GetFetchAfter() const;
    };
 
    /** \brief The name of this queue. */
