@@ -34,7 +34,6 @@
 #include <cstdlib>
 #include <iostream>
 #include <map>
-#include <regex>
 #include <set>
 #include <sstream>
 #include <string>
@@ -1515,9 +1514,6 @@ std::string GetProtectedKernelsRegex(pkgCache *cache, bool ReturnRemove)
    // needs to be initialized to 0s, might not be set up.
    utsname uts{};
    std::string bootedVersion;
-   std::string lastInstalledVersion;
-
-   std::string lastInstalledUname = _config->Find("APT::LastInstalledKernel");
 
    // Get currently booted version, but only when not on reproducible build.
    if (getenv("SOURCE_DATE_EPOCH") == 0)
@@ -1549,8 +1545,6 @@ std::string GetProtectedKernelsRegex(pkgCache *cache, bool ReturnRemove)
 
       if (pkgUname == uts.release)
 	 bootedVersion = pkgVersion;
-      if (pkgUname == lastInstalledUname)
-	 lastInstalledVersion = pkgVersion;
    }
 
    if (version2unames.size() == 0)
@@ -1568,40 +1562,37 @@ std::string GetProtectedKernelsRegex(pkgCache *cache, bool ReturnRemove)
 	 std::clog << "Keeping booted kernel " << bootedVersion << std::endl;
       keep.insert(bootedVersion);
    }
-   if (not lastInstalledVersion.empty())
-   {
-      if (Debug)
-	 std::clog << "Keeping installed kernel " << lastInstalledVersion << std::endl;
-      keep.insert(lastInstalledVersion);
-   }
    if (latest != version2unames.rend())
    {
       if (Debug)
 	 std::clog << "Keeping latest kernel " << latest->first << std::endl;
       keep.insert(latest->first);
    }
-   if (keep.size() < 3 && previous != version2unames.rend())
+   if (keep.size() < 2 && previous != version2unames.rend())
    {
       if (Debug)
 	 std::clog << "Keeping previous kernel " << previous->first << std::endl;
       keep.insert(previous->first);
    }
-
-   std::regex special("([\\.\\+])");
+   // Escape special characters '.' and '+' in version strings so we can build a regular expression
+   auto escapeSpecial = [](std::string input) -> std::string {
+      for (size_t pos = 0; (pos = input.find_first_of(".+", pos)) != input.npos; pos += 2) {
+	 input.insert(pos, 1, '\\');
+      }
+      return input;
+   };
    std::ostringstream ss;
    for (auto &pattern : _config->FindVector("APT::VersionedKernelPackages"))
    {
       // Legacy compatibility: Always protected the booted uname and last installed uname
-      if (not lastInstalledUname.empty())
-	 ss << "|^" << pattern << "-" << std::regex_replace(lastInstalledUname, special, "\\$1") << "$";
       if (*uts.release)
-	 ss << "|^" << pattern << "-" << std::regex_replace(uts.release, special, "\\$1") << "$";
+	 ss << "|^" << pattern << "-" << escapeSpecial(uts.release) << "$";
       for (auto const &kernel : version2unames)
       {
 	 if (ReturnRemove ? keep.find(kernel.first) == keep.end() : keep.find(kernel.first) != keep.end())
 	 {
 	    for (auto const &uname : kernel.second)
-	       ss << "|^" << pattern << "-" << std::regex_replace(uname, special, "\\$1") << "$";
+	       ss << "|^" << pattern << "-" << escapeSpecial(uname) << "$";
 	 }
       }
    }
