@@ -38,7 +38,7 @@ namespace APT {
 class PackageContainerInterface;
 class VersionContainerInterface;
 
-class CacheSetHelper {							/*{{{*/
+class APT_PUBLIC CacheSetHelper {							/*{{{*/
 /** \class APT::CacheSetHelper
     Simple base class with a lot of virtual methods which can be overridden
     to alter the behavior or the output of the CacheSets.
@@ -157,6 +157,8 @@ public:									/*{{{*/
 		}
 	}
 
+	std::string getLastVersionMatcher() const;
+	void setLastVersionMatcher(std::string const &matcher);
 									/*}}}*/
 protected:
 	bool ShowError;
@@ -167,6 +169,11 @@ protected:
 	pkgCache::VerIterator canNotGetCandInstVer(pkgCacheFile &Cache,
 		pkgCache::PkgIterator const &Pkg);
 
+	pkgCache::VerIterator canNotGetVerFromRelease(pkgCacheFile &Cache,
+		pkgCache::PkgIterator const &Pkg, std::string const &release);
+	pkgCache::VerIterator canNotGetVerFromVersionNumber(pkgCacheFile &Cache,
+		pkgCache::PkgIterator const &Pkg, std::string const &verstr);
+
 	bool PackageFromTask(PackageContainerInterface * const pci, pkgCacheFile &Cache, std::string pattern);
 	bool PackageFromRegEx(PackageContainerInterface * const pci, pkgCacheFile &Cache, std::string pattern);
 	bool PackageFromFnmatch(PackageContainerInterface * const pci, pkgCacheFile &Cache, std::string pattern);
@@ -191,7 +198,8 @@ private:
 	pkgCache::VerIterator canNotFindCandidateVer(pkgCacheFile &Cache, pkgCache::PkgIterator const &Pkg);
 	pkgCache::VerIterator canNotFindInstalledVer(pkgCacheFile &Cache, pkgCache::PkgIterator const &Pkg);
 
-	void * const d;
+	class Private;
+	Private * const d;
 };									/*}}}*/
 // Iterator templates for our Containers				/*{{{*/
 template<typename Interface, typename Master, typename iterator_type, typename container_iterator, typename container_value> class Container_iterator_base :
@@ -201,7 +209,7 @@ template<typename Interface, typename Master, typename iterator_type, typename c
 protected:
 	container_iterator _iter;
 public:
-	explicit Container_iterator_base(container_iterator i) : _iter(i) {}
+	explicit Container_iterator_base(container_iterator const &i) : _iter(i) {}
 	inline container_value operator*(void) const { return static_cast<iterator_type const*>(this)->getType(); };
 	operator container_iterator(void) const { return _iter; }
 	inline iterator_type& operator++() { ++_iter; return static_cast<iterator_type&>(*this); }
@@ -241,12 +249,10 @@ template<class Interface, class Container, class Master> class Container_iterato
 	typedef Container_iterator<Interface, Container, Master> iterator_type;
 	typedef typename Container::iterator container_iterator;
 public:
-	explicit Container_iterator(container_iterator i) :
+	explicit Container_iterator(container_iterator const &i) :
 	   Container_iterator_base<Interface, Master, iterator_type, container_iterator, typename Container::value_type>(i) {}
 
 	operator typename Master::const_iterator() { return typename Master::const_iterator(this->_iter); }
-	inline iterator_type& operator=(iterator_type const &i) { this->_iter = i._iter; return static_cast<iterator_type&>(*this); }
-	inline iterator_type& operator=(container_iterator const &i) { this->_iter = i; return static_cast<iterator_type&>(*this); }
 	inline typename Container::iterator::reference operator*(void) const { return *this->_iter; }
 
 	inline typename Container::value_type getType(void) const { return *this->_iter; }
@@ -279,7 +285,7 @@ public:
 	inline typename Container::value_type getType(void) const { return *this->_iter; }
 };
 									/*}}}*/
-class PackageContainerInterface {					/*{{{*/
+class APT_PUBLIC PackageContainerInterface {					/*{{{*/
 /** \class PackageContainerInterface
 
  * Interface ensuring that all operations can be executed on the yet to
@@ -325,6 +331,7 @@ public:
 	CacheSetHelper::PkgSelector getConstructor() const { return ConstructedBy; }
 	PackageContainerInterface();
 	explicit PackageContainerInterface(CacheSetHelper::PkgSelector const by);
+	PackageContainerInterface(PackageContainerInterface const &by);
 	PackageContainerInterface& operator=(PackageContainerInterface const &other);
 	virtual ~PackageContainerInterface();
 
@@ -333,7 +340,7 @@ private:
 	void * const d;
 };
 									/*}}}*/
-template<class Container> class PackageContainer : public PackageContainerInterface {/*{{{*/
+template<class Container> class APT_PUBLIC PackageContainer : public PackageContainerInterface {/*{{{*/
 /** \class APT::PackageContainer
 
     Simple wrapper around a container class like std::set to provide a similar
@@ -683,7 +690,7 @@ typedef PackageContainer<std::list<pkgCache::PkgIterator> > PackageList;
 typedef PackageContainer<std::deque<pkgCache::PkgIterator> > PackageDeque;
 typedef PackageContainer<std::vector<pkgCache::PkgIterator> > PackageVector;
 
-class VersionContainerInterface {					/*{{{*/
+class APT_PUBLIC VersionContainerInterface {					/*{{{*/
 /** \class APT::VersionContainerInterface
 
     Same as APT::PackageContainerInterface, just for Versions */
@@ -762,6 +769,7 @@ public:
 				   CacheSetHelper &helper);
 
 	VersionContainerInterface();
+	VersionContainerInterface(VersionContainerInterface const &other);
 	VersionContainerInterface& operator=(VersionContainerInterface const &other);
 	virtual ~VersionContainerInterface();
 private:
@@ -787,7 +795,7 @@ protected:								/*{{{*/
 									/*}}}*/
 };
 									/*}}}*/
-template<class Container> class VersionContainer : public VersionContainerInterface {/*{{{*/
+template<class Container> class APT_PUBLIC VersionContainer : public VersionContainerInterface {/*{{{*/
 /** \class APT::VersionContainer
 
     Simple wrapper around a container class like std::set to provide a similar
