@@ -144,6 +144,21 @@ static inline int isspace_ascii_inline(int const c)
    // 9='\t',10='\n',11='\v',12='\f',13='\r',32=' '
    return (c >= 9 && c <= 13) || c == ' ';
 }
+APT_PURE APT_HOT
+static inline int islower_ascii(int const c)
+{
+   return c >= 'a' && c <= 'z';
+}
+APT_PURE APT_HOT
+static inline int isupper_ascii(int const c)
+{
+   return c >= 'A' && c <= 'Z';
+}
+APT_PURE APT_HOT
+static inline int isalpha_ascii(int const c)
+{
+   return isupper_ascii(c) || islower_ascii(c);
+}
 
 APT_PUBLIC std::string StripEpoch(const std::string &VerStr);
 
