@@ -125,7 +125,6 @@ int pkgDepCache::DecreaseActionGroupLevel()
 struct pkgDepCache::Private
 {
    std::unique_ptr<InRootSetFunc> inRootSetFunc;
-   std::vector<bool> fullyExplored;
    std::unique_ptr<APT::CacheFilter::Matcher> IsAVersionedKernelPackage, IsProtectedKernelPackage;
 };
 pkgDepCache::pkgDepCache(pkgCache *const pCache, Policy *const Plcy) : group_level(0), Cache(pCache), PkgState(0), DepState(0),
@@ -895,8 +894,8 @@ static char const* PrintMode(char const mode)
 static bool IsModeChangeOk(pkgDepCache &Cache, pkgDepCache::ModeList const mode, pkgCache::PkgIterator const &Pkg,
 			   unsigned long const Depth, bool const FromUser, bool const DebugMarker)
 {
-   // we are not trying to hard…
-   if (unlikely(Depth > 100))
+   // we are not trying too hard…
+   if (unlikely(Depth > 3000))
       return false;
 
    // general sanity
@@ -931,6 +930,24 @@ static bool IsModeChangeOk(pkgDepCache &Cache, pkgDepCache::ModeList const mode,
 		   << " of " << APT::PrettyPkg(&Cache, Pkg) << std::endl;
       return false;
    }
+   // Do not allow removals of essential packages not explicitly triggered by the user
+   else if (mode == pkgDepCache::ModeDelete && (Pkg->Flags & pkgCache::Flag::Essential) == pkgCache::Flag::Essential &&
+	    not _config->FindB("APT::Get::Allow-Solver-Remove-Essential", false))
+   {
+      if (unlikely(DebugMarker == true))
+	 std::clog << OutputInDepth(Depth) << "Essential prevents Mark" << PrintMode(mode)
+		   << " of " << APT::PrettyPkg(&Cache, Pkg) << std::endl;
+      return false;
+   }
+   // Do not allow removals of essential packages not explicitly triggered by the user
+   else if (mode == pkgDepCache::ModeDelete && (Pkg->Flags & pkgCache::Flag::Important) == pkgCache::Flag::Important &&
+	    not _config->FindB("APT::Get::Allow-Solver-Remove-Essential", false))
+   {
+      if (unlikely(DebugMarker == true))
+	 std::clog << OutputInDepth(Depth) << "Protected prevents Mark" << PrintMode(mode)
+		   << " of " << APT::PrettyPkg(&Cache, Pkg) << std::endl;
+      return false;
+   }
 
    return true;
 }
@@ -1261,7 +1278,7 @@ bool pkgDepCache::MarkInstall_DiscardInstall(pkgCache::PkgIterator const &Pkg) /
 									/*}}}*/
 static bool MarkInstall_CollectDependencies(pkgDepCache const &Cache, pkgCache::VerIterator const &PV, std::vector<pkgCache::DepIterator> &toInstall, std::vector<pkgCache::DepIterator> &toRemove) /*{{{*/
 {
-   auto const propagateProctected = Cache[PV.ParentPkg()].Protect();
+   auto const propagateProtected = Cache[PV.ParentPkg()].Protect();
    for (auto Dep = PV.DependsList(); not Dep.end();)
    {
       auto const Start = Dep;
@@ -1273,7 +1290,7 @@ static bool MarkInstall_CollectDependencies(pkgDepCache const &Cache, pkgCache::
 	 if ((Cache[Dep] & pkgDepCache::DepInstall) == pkgDepCache::DepInstall)
 	    foundSolution = true;
       }
-      if (foundSolution && not propagateProctected)
+      if (foundSolution && not propagateProtected)
 	 continue;
 
       /* Check if this dep should be consider for install.
@@ -1335,10 +1352,10 @@ static APT::VersionVector getAllPossibleSolutions(pkgDepCache &Cache, pkgCache::
    return toUpgrade;
 }
 									/*}}}*/
-static bool MarkInstall_MarkDeleteForNotUpgradeable(pkgDepCache &Cache, bool const DebugAutoInstall, pkgCache::VerIterator const &PV, unsigned long const Depth, pkgCache::PkgIterator const &Pkg, bool const propagateProctected, APT::PackageVector &delayedRemove)/*{{{*/
+static bool MarkInstall_MarkDeleteForNotUpgradeable(pkgDepCache &Cache, bool const DebugAutoInstall, pkgCache::VerIterator const &PV, unsigned long const Depth, pkgCache::PkgIterator const &Pkg, bool const propagateProtected, APT::PackageVector &delayedRemove)/*{{{*/
 {
    auto &State = Cache[Pkg];
-   if (not propagateProctected)
+   if (not propagateProtected)
    {
       if (State.Delete())
 	 return true;
@@ -1363,7 +1380,7 @@ static bool MarkInstall_MarkDeleteForNotUpgradeable(pkgDepCache &Cache, bool con
    return true;
 }
 									/*}}}*/
-static bool MarkInstall_RemoveConflictsIfNotUpgradeable(pkgDepCache &Cache, bool const DebugAutoInstall, pkgCache::VerIterator const &PV, unsigned long Depth, std::vector<pkgCache::DepIterator> &toRemove, APT::PackageVector &toUpgrade, APT::PackageVector &delayedRemove, bool const propagateProctected, bool const FromUser) /*{{{*/
+static bool MarkInstall_RemoveConflictsIfNotUpgradeable(pkgDepCache &Cache, bool const DebugAutoInstall, pkgCache::VerIterator const &PV, unsigned long Depth, std::vector<pkgCache::DepIterator> &toRemove, APT::PackageVector &toUpgrade, APT::PackageVector &delayedRemove, bool const propagateProtected, bool const FromUser) /*{{{*/
 {
    /* Negative dependencies have no or-group
       If the candidate is effected try to keep current and discard candidate
@@ -1383,10 +1400,10 @@ static bool MarkInstall_RemoveConflictsIfNotUpgradeable(pkgDepCache &Cache, bool
 	    if (State.Install() && not Cache.MarkKeep(Pkg, false, false, Depth))
 	    {
 	       failedToRemoveSomething = true;
-	       if (not propagateProctected && not FromUser)
+	       if (not propagateProtected && not FromUser)
 		  break;
 	    }
-	    else if (propagateProctected)
+	    else if (propagateProtected)
 	    {
 	       MarkInstall_DiscardCandidate(Cache, Pkg);
 	       if (Pkg->CurrentVer == 0)
@@ -1395,14 +1412,14 @@ static bool MarkInstall_RemoveConflictsIfNotUpgradeable(pkgDepCache &Cache, bool
 	    else
 	       badCandidate.push_back(Pkg);
 	 }
-	 else if (not MarkInstall_MarkDeleteForNotUpgradeable(Cache, DebugAutoInstall, PV, Depth, Pkg, propagateProctected, delayedRemove))
+	 else if (not MarkInstall_MarkDeleteForNotUpgradeable(Cache, DebugAutoInstall, PV, Depth, Pkg, propagateProtected, delayedRemove))
 	 {
 	    failedToRemoveSomething = true;
-	    if (not propagateProctected && not FromUser)
+	    if (not propagateProtected && not FromUser)
 	       break;
 	 }
       }
-      if (failedToRemoveSomething && not propagateProctected && not FromUser)
+      if (failedToRemoveSomething && not propagateProtected && not FromUser)
 	 break;
       for (auto const &Ver : getAllPossibleSolutions(Cache, D, D, APT::CacheSetHelper::INSTALLED, true))
       {
@@ -1413,21 +1430,21 @@ static bool MarkInstall_RemoveConflictsIfNotUpgradeable(pkgDepCache &Cache, bool
 	    toUpgrade.push_back(Pkg);
 	 else if (State.CandidateVer == Pkg.CurrentVer())
 	    ; // already done in the first loop above
-	 else if (not MarkInstall_MarkDeleteForNotUpgradeable(Cache, DebugAutoInstall, PV, Depth, Pkg, propagateProctected, delayedRemove))
+	 else if (not MarkInstall_MarkDeleteForNotUpgradeable(Cache, DebugAutoInstall, PV, Depth, Pkg, propagateProtected, delayedRemove))
 	 {
 	    failedToRemoveSomething = true;
-	    if (not propagateProctected && not FromUser)
+	    if (not propagateProtected && not FromUser)
 	       break;
 	 }
       }
-      if (failedToRemoveSomething && not propagateProctected && not FromUser)
+      if (failedToRemoveSomething && not propagateProtected && not FromUser)
 	 break;
    }
    toRemove.clear();
    return not failedToRemoveSomething;
 }
 									/*}}}*/
-static bool MarkInstall_UpgradeOrRemoveConflicts(pkgDepCache &Cache, bool const DebugAutoInstall, unsigned long Depth, bool const ForceImportantDeps, APT::PackageVector &toUpgrade, bool const propagateProctected, bool const FromUser) /*{{{*/
+static bool MarkInstall_UpgradeOrRemoveConflicts(pkgDepCache &Cache, bool const DebugAutoInstall, unsigned long Depth, bool const ForceImportantDeps, APT::PackageVector &toUpgrade, bool const propagateProtected, bool const FromUser) /*{{{*/
 {
    bool failedToRemoveSomething = false;
    for (auto const &InstPkg : toUpgrade)
@@ -1438,17 +1455,17 @@ static bool MarkInstall_UpgradeOrRemoveConflicts(pkgDepCache &Cache, bool const
 	 if (not Cache.MarkDelete(InstPkg, false, Depth + 1, false))
 	 {
 	    failedToRemoveSomething = true;
-	    if (not propagateProctected && not FromUser)
+	    if (not propagateProtected && not FromUser)
 	       break;
 	 }
-	 else if (propagateProctected)
+	 else if (propagateProtected)
 	    Cache.MarkProtected(InstPkg);
       }
    toUpgrade.clear();
    return not failedToRemoveSomething;
 }
 									/*}}}*/
-static bool MarkInstall_InstallDependencies(pkgDepCache &Cache, bool const DebugAutoInstall, bool const DebugMarker, pkgCache::PkgIterator const &Pkg, unsigned long Depth, bool const ForceImportantDeps, std::vector<pkgCache::DepIterator> &toInstall, APT::PackageVector *const toMoveAuto, bool const propagateProctected, bool const FromUser) /*{{{*/
+static bool MarkInstall_InstallDependencies(pkgDepCache &Cache, bool const DebugAutoInstall, bool const DebugMarker, pkgCache::PkgIterator const &Pkg, unsigned long Depth, bool const ForceImportantDeps, std::vector<pkgCache::DepIterator> &toInstall, APT::PackageVector *const toMoveAuto, bool const propagateProtected, bool const FromUser) /*{{{*/
 {
    auto const IsSatisfiedByInstalled = [&](auto &D) { return (Cache[pkgCache::DepIterator{Cache, &D}] & pkgDepCache::DepInstall) == pkgDepCache::DepInstall; };
    bool failedToInstallSomething = false;
@@ -1458,7 +1475,7 @@ static bool MarkInstall_InstallDependencies(pkgDepCache &Cache, bool const Debug
       pkgCache::DepIterator Start, End;
       Dep.GlobOr(Start, End);
       bool foundSolution = std::any_of(Start, Dep, IsSatisfiedByInstalled);
-      if (foundSolution && not propagateProctected)
+      if (foundSolution && not propagateProtected)
 	 continue;
       bool const IsCriticalDep = Start.IsCritical();
       if (foundSolution)
@@ -1538,7 +1555,7 @@ static bool MarkInstall_InstallDependencies(pkgDepCache &Cache, bool const Debug
 	 if (DebugAutoInstall)
 	    std::clog << OutputInDepth(Depth) << "Installing " << InstPkg.FullName()
 		      << " as " << End.DepType() << " of " << Pkg.FullName() << '\n';
-	 if (propagateProctected && IsCriticalDep && possibleSolutions.size() == 1)
+	 if (propagateProtected && IsCriticalDep && possibleSolutions.size() == 1)
 	 {
 	    if (not Cache.MarkInstall(InstPkg, false, Depth + 1, false, ForceImportantDeps))
 	       continue;
@@ -1558,7 +1575,7 @@ static bool MarkInstall_InstallDependencies(pkgDepCache &Cache, bool const Debug
       if (not foundSolution && IsCriticalDep)
       {
 	 failedToInstallSomething = true;
-	 if (not propagateProctected && not FromUser)
+	 if (not propagateProtected && not FromUser)
 	    break;
       }
    }
@@ -1651,9 +1668,9 @@ bool pkgDepCache::MarkInstall(PkgIterator const &Pkg, bool AutoInst,
 	    P.iFlags &= (~Protected);
       }
       operator bool() noexcept { return already; }
-   } propagateProctected{PkgState[Pkg->ID]};
+   } propagateProtected{PkgState[Pkg->ID]};
 
-   if (not MarkInstall_UpgradeOrRemoveConflicts(*this, DebugAutoInstall, Depth, ForceImportantDeps, toUpgrade, propagateProctected, FromUser))
+   if (not MarkInstall_UpgradeOrRemoveConflicts(*this, DebugAutoInstall, Depth, ForceImportantDeps, toUpgrade, propagateProtected, FromUser))
    {
       if (failEarly)
       {
@@ -1681,7 +1698,7 @@ bool pkgDepCache::MarkInstall(PkgIterator const &Pkg, bool AutoInst,
 
    APT::PackageVector toMoveAuto;
    if (not MarkInstall_InstallDependencies(*this, DebugAutoInstall, DebugMarker, Pkg, Depth, ForceImportantDeps, toInstall,
-					   MoveAutoBitToDependencies ? &toMoveAuto : nullptr, propagateProctected, FromUser))
+					   MoveAutoBitToDependencies ? &toMoveAuto : nullptr, propagateProtected, FromUser))
    {
       if (failEarly)
       {
@@ -1794,7 +1811,7 @@ bool pkgDepCache::IsInstallOkDependenciesSatisfiableByCandidates(PkgIterator con
       DepIterator Start = Dep;
       bool foundSolution = false;
       unsigned Ors = 0;
-      // Is it possible to statisfy this dependency?
+      // Is it possible to satisfy this dependency?
       for (bool LastOR = true; not Dep.end() && LastOR; ++Dep, ++Ors)
       {
 	 LastOR = (Dep->CompareOp & Dep::Or) == Dep::Or;
@@ -2039,7 +2056,7 @@ bool pkgDepCache::SetCandidateRelease(pkgCache::VerIterator TargetVer,
 	 if (itsFine == false)
 	 {
 	    // change the candidate
-	    Changed.push_back(make_pair(V, TargetVer));
+	    Changed.emplace_back(V, TargetVer);
 	    if (SetCandidateRelease(V, TargetRel, Changed) == false)
 	    {
 	       if (stillOr == false)
@@ -2201,18 +2218,17 @@ pkgDepCache::InRootSetFunc *pkgDepCache::GetCachedRootSetFunc()
    return d->inRootSetFunc.get();
 }
 									/*}}}*/
-bool pkgDepCache::MarkFollowsRecommends()
+bool pkgDepCache::MarkFollowsRecommends()				/*{{{*/
 {
   return _config->FindB("APT::AutoRemove::RecommendsImportant", true);
 }
-
-bool pkgDepCache::MarkFollowsSuggests()
+									/*}}}*/
+bool pkgDepCache::MarkFollowsSuggests()					/*{{{*/
 {
   return _config->FindB("APT::AutoRemove::SuggestsImportant", true);
 }
-
-// pkgDepCache::MarkRequired - the main mark algorithm			/*{{{*/
-static bool IsPkgInBoringState(pkgCache::PkgIterator const &Pkg, pkgDepCache::StateCache const * const PkgState)
+									/*}}}*/
+static bool IsPkgInBoringState(pkgCache::PkgIterator const &Pkg, pkgDepCache::StateCache const * const PkgState)/*{{{*/
 {
    if (Pkg->CurrentVer == 0)
    {
@@ -2225,82 +2241,37 @@ static bool IsPkgInBoringState(pkgCache::PkgIterator const &Pkg, pkgDepCache::St
 	 return true;
    }
    return false;
-}
-bool pkgDepCache::MarkRequired(InRootSetFunc &userFunc)
-{
-   if (_config->Find("APT::Solver", "internal") != "internal")
-      return true;
-
-   bool const debug_autoremove = _config->FindB("Debug::pkgAutoRemove",false);
-
-   // init the states
-   auto const PackagesCount = Head().PackageCount;
-   for(auto i = decltype(PackagesCount){0}; i < PackagesCount; ++i)
-   {
-      PkgState[i].Marked  = false;
-      PkgState[i].Garbage = false;
-   }
-   d->fullyExplored = std::vector<bool>(PackagesCount, false);
-   if (debug_autoremove)
-      for(PkgIterator p = PkgBegin(); !p.end(); ++p)
-	 if(PkgState[p->ID].Flags & Flag::Auto)
-	    std::clog << "AutoDep: " << p.FullName() << std::endl;
-
-   bool const follow_recommends = MarkFollowsRecommends();
-   bool const follow_suggests   = MarkFollowsSuggests();
-
-   // do the mark part, this is the core bit of the algorithm
-   for (PkgIterator P = PkgBegin(); !P.end(); ++P)
-   {
-      if (PkgState[P->ID].Marked || IsPkgInBoringState(P, PkgState))
-	 continue;
-
-      const char *reason = nullptr;
-
-      if ((PkgState[P->ID].Flags & Flag::Auto) == 0)
-	 reason = "Manual-Installed";
-      else if (P->Flags & Flag::Essential)
-	 reason = "Essential";
-      else if (P->Flags & Flag::Important)
-	 reason = "Important";
-      else if (P->CurrentVer != 0 && P.CurrentVer()->Priority == pkgCache::State::Required)
-	 reason = "Required";
-      else if (userFunc.InRootSet(P))
-	 reason = "Blacklisted [APT::NeverAutoRemove]";
-      else if (not IsModeChangeOk(*this, ModeGarbage, P, 0, false, DebugMarker))
-	 reason = "Hold";
-      else
-	 continue;
-
-      if (PkgState[P->ID].Install())
-	 MarkPackage(P, PkgState[P->ID].InstVerIter(*this),
-	       follow_recommends, follow_suggests, reason);
-      else
-	 MarkPackage(P, P.CurrentVer(),
-	       follow_recommends, follow_suggests, reason);
-   }
-   d->fullyExplored.clear();
-   return true;
 }
 									/*}}}*/
 // MarkPackage - mark a single package in Mark-and-Sweep		/*{{{*/
-void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
-			      const pkgCache::VerIterator &Ver,
-			      bool const &follow_recommends,
-			      bool const &follow_suggests,
-			      const char *reason)
+static bool MarkPackage(pkgCache::PkgIterator const &Pkg,
+			pkgCache::VerIterator const &Ver,
+			bool const follow_recommends,
+			bool const follow_suggests,
+			bool const debug_autoremove,
+			std::string_view const reason,
+			size_t const Depth,
+			pkgCache &Cache,
+			pkgDepCache &DepCache,
+			pkgDepCache::StateCache *const PkgState,
+			std::vector<bool> &fullyExplored,
+			std::unique_ptr<APT::CacheFilter::Matcher> &IsAVersionedKernelPackage,
+			std::unique_ptr<APT::CacheFilter::Matcher> &IsProtectedKernelPackage)
 {
-   if (Ver.end() || (d->fullyExplored[Pkg->ID] && PkgState[Pkg->ID].Marked))
-      return;
+   if (Ver.end() || PkgState[Pkg->ID].Marked)
+      return true;
 
    if (IsPkgInBoringState(Pkg, PkgState))
    {
-      d->fullyExplored[Pkg->ID] = true;
-      return;
+      fullyExplored[Pkg->ID] = true;
+      return true;
    }
 
+   // we are not trying too hard…
+   if (unlikely(Depth > 3000))
+      return false;
+
    PkgState[Pkg->ID].Marked = true;
-   bool const debug_autoremove = _config->FindB("Debug::pkgAutoRemove", false);
    if(debug_autoremove)
       std::clog << "Marking: " << Pkg.FullName() << " " << Ver.VerStr()
 		<< " (" << reason << ")" << std::endl;
@@ -2308,20 +2279,20 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
    auto const sort_by_source_version = [](pkgCache::VerIterator const &A, pkgCache::VerIterator const &B) {
       auto const verret = A.Cache()->VS->CmpVersion(A.SourceVerStr(), B.SourceVerStr());
       if (verret != 0)
-	 return verret > 0;
+	 return verret < 0;
       return A->ID < B->ID;
    };
 
    for (auto D = Ver.DependsList(); not D.end(); ++D)
    {
       auto const T = D.TargetPkg();
-      if (T.end() || d->fullyExplored[T->ID])
+      if (T.end() || fullyExplored[T->ID])
 	 continue;
 
-      if (D->Type != Dep::Depends &&
-	    D->Type != Dep::PreDepends &&
-	    (follow_recommends == false || D->Type != Dep::Recommends) &&
-	    (follow_suggests == false || D->Type != Dep::Suggests))
+      if (D->Type != pkgCache::Dep::Depends &&
+	    D->Type != pkgCache::Dep::PreDepends &&
+	    (not follow_recommends || D->Type != pkgCache::Dep::Recommends) &&
+	    (not follow_suggests || D->Type != pkgCache::Dep::Suggests))
 	 continue;
 
       bool unsatisfied_choice = false;
@@ -2329,7 +2300,7 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
       // collect real part
       if (not IsPkgInBoringState(T, PkgState))
       {
-	 auto const TV = (PkgState[T->ID].Install()) ? PkgState[T->ID].InstVerIter(*this) : T.CurrentVer();
+	 auto const TV = (PkgState[T->ID].Install()) ? PkgState[T->ID].InstVerIter(DepCache) : T.CurrentVer();
 	 if (likely(not TV.end()))
 	 {
 	    if (not D.IsSatisfied(TV))
@@ -2349,7 +2320,7 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
 
 	 // we want to ignore provides from uninteresting versions
 	 auto const PV = (PkgState[PP->ID].Install()) ?
-	    PkgState[PP->ID].InstVerIter(*this) : PP.CurrentVer();
+	    PkgState[PP->ID].InstVerIter(DepCache) : PP.CurrentVer();
 	 if (unlikely(PV.end()) || PV != Prv.OwnerVer())
 	    continue;
 
@@ -2361,16 +2332,13 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
       // only latest binary package of a source package is marked instead of all
       for (auto &providers : providers_by_source)
       {
-	 std::sort(providers.second.begin(), providers.second.end(), sort_by_source_version);
-	 auto const highestSrcVer = (*providers.second.begin()).SourceVerStr();
-	 auto notThisVer = std::find_if_not(providers.second.begin(), providers.second.end(), [&](auto const &V) { return strcmp(highestSrcVer, V.SourceVerStr()) == 0; });
-	 if (notThisVer != providers.second.end())
-	    providers.second.erase(notThisVer, providers.second.end());
+	 auto const highestSrcVer = (*std::max_element(providers.second.begin(), providers.second.end(), sort_by_source_version)).SourceVerStr();
+	 providers.second.erase(std::remove_if(providers.second.begin(), providers.second.end(), [&](auto const &V) { return strcmp(highestSrcVer, V.SourceVerStr()) != 0; }), providers.second.end());
 	 // if the provider is a versioned kernel package mark them only for protected kernels
 	 if (providers.second.size() == 1)
 	    continue;
-	 if (not d->IsAVersionedKernelPackage)
-	    d->IsAVersionedKernelPackage = [&]() -> std::unique_ptr<APT::CacheFilter::Matcher> {
+	 if (not IsAVersionedKernelPackage)
+	    IsAVersionedKernelPackage = [&]() -> std::unique_ptr<APT::CacheFilter::Matcher> {
 	       auto const patterns = _config->FindVector("APT::VersionedKernelPackages");
 	       if (patterns.empty())
 		  return std::make_unique<APT::CacheFilter::FalseMatcher>();
@@ -2380,33 +2348,92 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
 	       regex << patterns.back() << "-.*$";
 	       return std::make_unique<APT::CacheFilter::PackageNameMatchesRegEx>(regex.str());
 	    }();
-	 if (not std::all_of(providers.second.begin(), providers.second.end(), [&](auto const &Prv) { return (*d->IsAVersionedKernelPackage)(Prv.ParentPkg()); }))
+	 if (not std::all_of(providers.second.begin(), providers.second.end(), [&](auto const &Prv) { return (*IsAVersionedKernelPackage)(Prv.ParentPkg()); }))
 	    continue;
 	 // … if there is at least one for protected kernels installed
-	 if (not d->IsProtectedKernelPackage)
-	    d->IsProtectedKernelPackage = APT::KernelAutoRemoveHelper::GetProtectedKernelsFilter(Cache);
-	 if (not std::any_of(providers.second.begin(), providers.second.end(), [&](auto const &Prv) { return (*d->IsProtectedKernelPackage)(Prv.ParentPkg()); }))
+	 if (not IsProtectedKernelPackage)
+	    IsProtectedKernelPackage = APT::KernelAutoRemoveHelper::GetProtectedKernelsFilter(&Cache);
+	 if (not std::any_of(providers.second.begin(), providers.second.end(), [&](auto const &Prv) { return (*IsProtectedKernelPackage)(Prv.ParentPkg()); }))
 	    continue;
 	 providers.second.erase(std::remove_if(providers.second.begin(), providers.second.end(),
-					       [&](auto const &Prv) { return not((*d->IsProtectedKernelPackage)(Prv.ParentPkg())); }),
+					       [&](auto const &Prv) { return not((*IsProtectedKernelPackage)(Prv.ParentPkg())); }),
 				providers.second.end());
       }
 
       if (not unsatisfied_choice)
-	 d->fullyExplored[T->ID] = true;
+	 fullyExplored[T->ID] = true;
       for (auto const &providers : providers_by_source)
       {
 	 for (auto const &PV : providers.second)
 	 {
 	    auto const PP = PV.ParentPkg();
 	    if (debug_autoremove)
-	       std::clog << "Following dep: " << APT::PrettyDep(this, D)
+	       std::clog << "Following dep: " << APT::PrettyDep(&DepCache, D)
 			 << ", provided by " << PP.FullName() << " " << PV.VerStr()
 			 << " (" << providers_by_source.size() << "/" << providers.second.size() << ")\n";
-	    MarkPackage(PP, PV, follow_recommends, follow_suggests, "Dependency");
+	    if (not MarkPackage(PP, PV, follow_recommends, follow_suggests, debug_autoremove,
+				"Dependency", Depth + 1, Cache, DepCache, PkgState, fullyExplored,
+				IsAVersionedKernelPackage, IsProtectedKernelPackage))
+	       return false;
 	 }
       }
    }
+   return true;
+}
+									/*}}}*/
+// pkgDepCache::MarkRequired - the main mark algorithm			/*{{{*/
+bool pkgDepCache::MarkRequired(InRootSetFunc &userFunc)
+{
+   if (_config->Find("APT::Solver", "internal") != "internal")
+      return true;
+
+   // init the states
+   auto const PackagesCount = Head().PackageCount;
+   for(auto i = decltype(PackagesCount){0}; i < PackagesCount; ++i)
+   {
+      PkgState[i].Marked  = false;
+      PkgState[i].Garbage = false;
+   }
+   std::vector<bool> fullyExplored(PackagesCount, false);
+
+   bool const debug_autoremove = _config->FindB("Debug::pkgAutoRemove", false);
+   if (debug_autoremove)
+      for(PkgIterator p = PkgBegin(); !p.end(); ++p)
+	 if(PkgState[p->ID].Flags & Flag::Auto)
+	    std::clog << "AutoDep: " << p.FullName() << std::endl;
+
+   bool const follow_recommends = MarkFollowsRecommends();
+   bool const follow_suggests   = MarkFollowsSuggests();
+
+   // do the mark part, this is the core bit of the algorithm
+   for (PkgIterator P = PkgBegin(); !P.end(); ++P)
+   {
+      if (PkgState[P->ID].Marked || IsPkgInBoringState(P, PkgState))
+	 continue;
+
+      std::string_view reason;
+      if ((PkgState[P->ID].Flags & Flag::Auto) == 0)
+	 reason = "Manual-Installed";
+      else if (P->Flags & Flag::Essential)
+	 reason = "Essential";
+      else if (P->Flags & Flag::Important)
+	 reason = "Important";
+      else if (P->CurrentVer != 0 && P.CurrentVer()->Priority == pkgCache::State::Required)
+	 reason = "Required";
+      else if (userFunc.InRootSet(P))
+	 reason = "Blacklisted [APT::NeverAutoRemove]";
+      else if (not IsModeChangeOk(*this, ModeGarbage, P, 0, false, DebugMarker))
+	 reason = "Hold";
+      else
+	 continue;
+
+      pkgCache::VerIterator const PV = (PkgState[P->ID].Install()) ? PkgState[P->ID].InstVerIter(*this) : P.CurrentVer();
+      if (not MarkPackage(P, PV, follow_recommends, follow_suggests, debug_autoremove,
+			  reason, 0, *Cache, *this, PkgState, fullyExplored,
+			  d->IsAVersionedKernelPackage, d->IsProtectedKernelPackage))
+	 return false;
+   }
+   return true;
 }
 									/*}}}*/
 bool pkgDepCache::Sweep()						/*{{{*/
