@@ -304,19 +304,7 @@ bool InstallPackages(CacheFile &Cache, APT::PackageVector &HeldBackPackages, boo
       if (_config->FindB("APT::Get::Trivial-Only",false) == true)
 	 return _error->Error(_("Trivial Only specified but this is not a trivial operation."));
 
-      // TRANSLATOR: This string needs to be typed by the user as a confirmation, so be
-      //             careful with hard to type or special characters (like non-breaking spaces)
-      const char *Prompt = _("Yes, do as I say!");
-      std::string question;
-      strprintf(question,
-	       _("You are about to do something potentially harmful.\n"
-		 "To continue type in the phrase '%s'\n"
-		 " ?] "),Prompt);
-      if (AnalPrompt(question, Prompt) == false)
-      {
-	 c2out << _("Abort.") << std::endl;
-	 exit(1);
-      }
+      return _error->Error(_("Removing essential system-critical packages is not permitted. This might break the system."));
    }
    else
    {
@@ -704,7 +692,7 @@ bool DoCacheManipulationFromCommandLine(CommandLine &CmdL, std::vector<PseudoPkg
 
       if (Fix != NULL && _config->FindB("APT::Get::AutoSolving", true) == true)
       {
-	 InstallAction.propergateReleaseCandiateSwitching(helper.selectedByRelease, c0out);
+	 InstallAction.propagateReleaseCandidateSwitching(helper.selectedByRelease, c0out);
 	 InstallAction.doAutoInstall();
       }
 
@@ -860,6 +848,8 @@ bool DoInstall(CommandLine &CmdL)
 {
    CacheFile Cache;
    Cache.InhibitActionGroups(true);
+   if (Cache.BuildSourceList() == false)
+      return false;
    auto VolatileCmdL = GetPseudoPackages(Cache.GetSourceList(), CmdL, AddVolatileBinaryFile, "");
 
    // then open the cache
@@ -1062,7 +1052,7 @@ void TryToInstall::operator() (pkgCache::VerIterator const &Ver) {
    }
 }
 									/*}}}*/
-bool TryToInstall::propergateReleaseCandiateSwitching(std::list<std::pair<pkgCache::VerIterator, std::string> > const &start, std::ostream &out)/*{{{*/
+bool TryToInstall::propagateReleaseCandidateSwitching(std::list<std::pair<pkgCache::VerIterator, std::string> > const &start, std::ostream &out)/*{{{*/
 {
    for (std::list<std::pair<pkgCache::VerIterator, std::string> >::const_iterator s = start.begin();
 	 s != start.end(); ++s)
