@@ -75,3 +75,31 @@ TEST(ArchitecturesTest,Duplicates)
 
    _config->Clear();
 }
+TEST(ArchitecturesTest,VeryForeign)
+{
+   _config->Clear();
+   _config->Set("APT::Architectures::", "i386");
+   _config->Set("APT::Architectures::", "amd64");
+   _config->Set("APT::Architectures::", "armel");
+
+   auto vec = APT::Configuration::getArchitectures(false);
+   ASSERT_EQ(3u, vec.size());
+   EXPECT_EQ("i386", vec[0]);
+   EXPECT_EQ("amd64", vec[1]);
+   EXPECT_EQ("armel", vec[2]);
+
+   _config->Set("APT::BarbarianArchitectures::", "mipsel");
+   vec = APT::Configuration::getArchitectures(false);
+   ASSERT_EQ(3u, vec.size());
+   EXPECT_EQ("i386", vec[0]);
+   EXPECT_EQ("amd64", vec[1]);
+   EXPECT_EQ("armel", vec[2]);
+
+   _config->Set("APT::BarbarianArchitectures::", "armel");
+   vec = APT::Configuration::getArchitectures(false);
+   ASSERT_EQ(2u, vec.size());
+   EXPECT_EQ("i386", vec[0]);
+   EXPECT_EQ("amd64", vec[1]);
+
+   _config->Clear();
+}
